/*
    UHU-menu -- output plugin for Sawfish (2.0)
    Copyright (C) 2003  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i <= n; i++) fprintf(f, "\t");
}

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		if (*s == '\\' || *s == '\"') fputc('\\', f);
		else if (*s == '_') fputc('_', f);
		fputc(*s, f);
		s++;
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s == '%') {
			s++;
			if (*s == '%' || *s == '\0') fputc('%', f);
			if (*s != '\0') s++;
			continue;
		}
		if (*s == '\\' || *s == '\"') fputc('\\', f);
		fputc(*s, f);
		s++;
	}
}

static void dump (FILE *f, const struct item *item, const char *lang, int n)
{
	int i;
	int major;
	if (!item->issubmenu) {
		indent(f, n);
		fprintf(f, "(\"");
		print_name(f, get_value(item, NULL, "Name", lang));
		fprintf(f, "\" (system \"");
		if (item->terminal) fprintf(f, "terminal ");
		print_exec(f, get_value(item, NULL, "Exec", lang));
		fprintf(f, " &\"))\n");
	} else {
		if (n > 0) {
			indent(f, n);
			fprintf(f, "(\"");
			print_name(f, get_value(item, NULL, "Name", lang));
			fprintf(f, "\". (\n");
		}
		major = 0;
		for (i = 0; i < item->subitems_nr; i++) {
			if (i > 0 && item->subitems[i].major != major) {
				indent(f, n+1);
				fprintf(f, "()\n");
			}
			dump(f, &item->subitems[i], lang, n+1);
			major = item->subitems[i].major;
		}
		if (n > 0) {
			indent(f, n);
			fprintf(f, "))\n");
		}
	}
}

int write_menu_lang (const struct item *item, const char *path,
  const struct language *language, struct keep_files *keep_files)
{
	char *lang;
	char file[PATH_MAX];
	char file_tmp[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu%s%s.jl", path, lang[0] ? "." : "", lang);
	sprintf(file_tmp, "%s..tmp", file);
	if ((f = fopen(file_tmp, "w")) == NULL) {
		message_e(3, "fopen %s", file_tmp);
		return -1;
	}
	fprintf(f, "(define-structure sawfish.wm.uhumenu\n");
	fprintf(f, "\t(export uhu-menu)\n");
	fprintf(f, "\t(open rep)\n");
	fprintf(f, "\t(eval-when-compile (require 'rep.io.timers))\n");
	fprintf(f, "\t(defvar uhu-menu '(\n");
	dump(f, item, lang, 0);
	fprintf(f, "\t))\n");
	fprintf(f, ")\n");
	fclose(f);
	move(file_tmp, file);
	keep_file(keep_files, strrchr(file, '/') + 1);
	return 0;
}

